@@ -1,11 +1,11 @@
 /*
- * $Id: SphinxClient.java 3144 2012-03-12 08:58:04Z tomat $
+ * $Id: SphinxClient.java 4885 2015-01-20 07:02:07Z deogar $
  *
  * Java version of Sphinx searchd client (Java API)
  *
  * Copyright (c) 2007, Vladimir Fedorkov
- * Copyright (c) 2007-2012, Andrew Aksyonoff
- * Copyright (c) 2008-2012, Sphinx Technologies Inc
+ * Copyright (c) 2007-2015, Andrew Aksyonoff
+ * Copyright (c) 2008-2015, Sphinx Technologies Inc
  * All rights reserved
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,6 +14,11 @@
  * did not, you can find it at http://www.gnu.org/
  */
 
+// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+//							WARNING
+// We strongly recommend you to use SphinxQL instead of the API
+// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+
 package org.sphx.api;
 
 import java.io.*;
@@ -90,7 +95,7 @@ public class SphinxClient
 	private final static int VER_MAJOR_PROTO		= 0x1;
 	private final static int VER_COMMAND_SEARCH		= 0x119;
 	private final static int VER_COMMAND_EXCERPT	= 0x102;
-	private final static int VER_COMMAND_UPDATE		= 0x102;
+	private final static int VER_COMMAND_UPDATE		= 0x103;
 	private final static int VER_COMMAND_KEYWORDS	= 0x100;
 	private final static int VER_COMMAND_FLUSHATTRS	= 0x100;
 
@@ -160,7 +165,7 @@ public class SphinxClient
 
 		_offset	= 0;
 		_limit	= 20;
-		_mode	= SPH_MATCH_ALL;
+		_mode	= SPH_MATCH_EXTENDED2;
 		_sort	= SPH_SORT_RELEVANCE;
 		_sortby	= "";
 		_minId	= 0;
@@ -492,9 +497,10 @@ public class SphinxClient
 		_maxQueryTime = maxTime;
 	}
 
-	/** Set matching mode. */
+	/** Set matching mode. DEPRECATED */
 	public void SetMatchMode(int mode) throws SphinxException
 	{
+		System.out.println ( "DEPRECATED: Do not call this method or, even better, use SphinxQL instead of an API\n" );
 		myAssert (
 			mode==SPH_MATCH_ALL ||
 			mode==SPH_MATCH_ANY ||
@@ -726,11 +732,12 @@ public class SphinxClient
 	}
 
 	/**
-	 * Set attribute values override (one override list per attribute).
+	 * DEPRECATED: Set attribute values override (one override list per attribute).
 	 * @param values maps Long document IDs to Int/Long/Float values (as specified in attrtype).
 	 */
 	public void SetOverride ( String attrname, int attrtype, Map values ) throws SphinxException
 	{
+		System.out.println ( "DEPRECATED: Do not call this method. Use SphinxQL REMAP() function instead.\n" );
 		myAssert ( attrname!=null && attrname.length()>0, "attrname must not be empty" );
 		myAssert ( attrtype==SPH_ATTR_INTEGER || attrtype==SPH_ATTR_TIMESTAMP || attrtype==SPH_ATTR_BOOL || attrtype==SPH_ATTR_FLOAT || attrtype==SPH_ATTR_BIGINT,
 			"unsupported attrtype (must be one of INTEGER, TIMESTAMP, BOOL, FLOAT, or BIGINT)" );
@@ -1219,11 +1226,12 @@ public class SphinxClient
 	 * @param attrs		array with the names of the attributes to update
 	 * @param values	array of updates; each long[] entry must contains document ID
 	 *					in the first element, and all new attribute values in the following ones
+	 * @param ignorenonexistent	the flag whether to silently ignore non existent columns up update request
 	 * @return			-1 on failure, amount of actually found and updated documents (might be 0) on success
 	 *
 	 * @throws			SphinxException on invalid parameters
 	 */
-	public int UpdateAttributes ( String index, String[] attrs, long[][] values ) throws SphinxException
+	public int UpdateAttributes ( String index, String[] attrs, long[][] values, boolean ignorenonexistent ) throws SphinxException
 	{
 		/* check args */
 		myAssert ( index!=null && index.length()>0, "no index name provided" );
@@ -1243,6 +1251,7 @@ public class SphinxClient
 			writeNetUTF8 ( req, index );
 
 			req.writeInt ( attrs.length );
+			req.writeInt ( ignorenonexistent ? 1 : 0 );
 			for ( int i=0; i<attrs.length; i++ )
 			{
 				writeNetUTF8 ( req, attrs[i] );
@@ -1301,11 +1310,12 @@ public class SphinxClient
 	 * @param docid		id of document to update
 	 * @param attrs		array with the names of the attributes to update
 	 * @param values		array of updates; each int[] entry must contains all new attribute values
+	 * @param ignorenonexistent	the flag whether to silently ignore non existent columns up update request
 	 * @return			-1 on failure, amount of actually found and updated documents (might be 0) on success
 	 *
 	 * @throws			SphinxException on invalid parameters
 	 */
-	public int UpdateAttributesMVA ( String index, long docid, String[] attrs, int[][] values ) throws SphinxException
+	public int UpdateAttributesMVA ( String index, long docid, String[] attrs, int[][] values, boolean ignorenonexistent ) throws SphinxException
 	{
 		/* check args */
 		myAssert ( index!=null && index.length()>0, "no index name provided" );
@@ -1326,6 +1336,7 @@ public class SphinxClient
 			writeNetUTF8 ( req, index );
 
 			req.writeInt ( attrs.length );
+			req.writeInt ( ignorenonexistent ? 1 : 0 );
 			for ( int i=0; i<attrs.length; i++ )
 			{
 				writeNetUTF8 ( req, attrs[i] );
@@ -1365,7 +1376,15 @@ public class SphinxClient
 		}
 	}
 	
+	public int UpdateAttributes ( String index, String[] attrs, long[][] values ) throws SphinxException
+	{
+		return UpdateAttributes ( index, attrs, values, false );
+	}
 
+	public int UpdateAttributesMVA ( String index, long docid, String[] attrs, int[][] values ) throws SphinxException
+	{
+		return UpdateAttributesMVA ( index, docid, attrs, values, false );
+	}
 
 	/**
      * Connect to searchd server, and generate keyword list for a given query.
@@ -1507,5 +1526,5 @@ public class SphinxClient
 }
 
 /*
- * $Id: SphinxClient.java 3144 2012-03-12 08:58:04Z tomat $
+ * $Id: SphinxClient.java 4885 2015-01-20 07:02:07Z deogar $
  */
