@@ -1,10 +1,10 @@
 //
-// $Id: spelldump.cpp 3087 2012-01-30 23:07:35Z shodan $
+// $Id: spelldump.cpp 4885 2015-01-20 07:02:07Z deogar $
 //
 
 //
-// Copyright (c) 2001-2012, Andrew Aksyonoff
-// Copyright (c) 2008-2012, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -293,17 +293,17 @@ bool CISpellAffixRule::CheckSuffix ( const CSphString & sWord ) const
 	int iCondI = m_iCondLen-1;
 	for ( int i=m_iWordLen-1; iCondI>=0 && i>=0; --i )
 	{
-		if ( m_sCondition.cstr()[iCondI]!=']' )
+		if ( m_sCondition.cstr()[iCondI]=='.' )
+		{
+			--iCondI;
+		} else if ( m_sCondition.cstr()[iCondI]!=']' )
 		{
 			if ( m_sCondition.cstr()[iCondI]!=sWord.cstr()[i] )
 				return false;
-
 			--iCondI;
-
 		} else
 		{
 			int iRangeStart = -1;
-
 			for ( int j=iCondI; j>=0 && iRangeStart==-1; --j )
 				if ( m_sCondition.cstr()[j]=='[' )
 					iRangeStart = j;
@@ -314,12 +314,10 @@ bool CISpellAffixRule::CheckSuffix ( const CSphString & sWord ) const
 			{
 				if ( !IsInSet ( sWord.cstr () [i], m_sCondition.SubString ( iRangeStart + 1, iCondI - iRangeStart - 1 ).cstr () ) )
 					return false;
-
 				iCondI = iRangeStart - 1;
 			}
 		}
 	}
-
 	return true;
 }
 
@@ -1191,5 +1189,5 @@ int main ( int iArgs, char ** dArgs )
 }
 
 //
-// $Id: spelldump.cpp 3087 2012-01-30 23:07:35Z shodan $
+// $Id: spelldump.cpp 4885 2015-01-20 07:02:07Z deogar $
 //
