@@ -1,10 +1,10 @@
 //
-// $Id: sphinxrt.h 3125 2012-02-28 15:39:55Z shodan $
+// $Id: sphinxrt.h 4885 2015-01-20 07:02:07Z deogar $
 //
 
 //
-// Copyright (c) 2001-2012, Andrew Aksyonoff
-// Copyright (c) 2008-2012, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -18,30 +18,34 @@
 
 #include "sphinx.h"
 #include "sphinxutils.h"
+#include "sphinxstem.h"
+
+struct CSphReconfigureSettings;
+struct CSphReconfigureSetup;
 
 /// RAM based updateable backend interface
 class ISphRtIndex : public CSphIndex
 {
 public:
-	explicit ISphRtIndex ( const char * sIndexName, const char * sName ) : CSphIndex ( sIndexName, sName ) {}
+	explicit ISphRtIndex ( const char * sIndexName, const char * sFileName ) : CSphIndex ( sIndexName, sFileName ) {}
 
 	/// get internal schema (to use for Add calls)
 	virtual const CSphSchema & GetInternalSchema () const { return m_tSchema; }
 
 	/// insert/update document in current txn
 	/// fails in case of two open txns to different indexes
-	virtual bool AddDocument ( int iFields, const char ** ppFields, const CSphMatch & tDoc, bool bReplace, const char ** ppStr, const CSphVector<DWORD> & dMvas, CSphString & sError ) = 0;
+	virtual bool AddDocument ( int iFields, const char ** ppFields, const CSphMatch & tDoc, bool bReplace, const CSphString & sTokenFilterOptions, const char ** ppStr, const CSphVector<DWORD> & dMvas, CSphString & sError, CSphString & sWarning ) = 0;
 
 	/// insert/update document in current txn
 	/// fails in case of two open txns to different indexes
-	virtual bool AddDocument ( ISphHits * pHits, const CSphMatch & tDoc, const char ** ppStr, const CSphVector<DWORD> & dMvas, CSphString & sError ) = 0;
+	virtual bool AddDocument ( ISphHits * pHits, const CSphMatch & tDoc, const char ** ppStr, const CSphVector<DWORD> & dMvas, CSphString & sError, CSphString & sWarning ) = 0;
 
 	/// delete document in current txn
 	/// fails in case of two open txns to different indexes
 	virtual bool DeleteDocument ( const SphDocID_t * pDocs, int iDocs, CSphString & sError ) = 0;
 
 	/// commit pending changes
-	virtual void Commit () = 0;
+	virtual void Commit ( int * pDeleted=NULL ) = 0;
 
 	/// undo pending changes
 	virtual void RollBack () = 0;
@@ -57,11 +61,26 @@ public:
 
 	/// attach a disk chunk to current index
 	virtual bool AttachDiskIndex ( CSphIndex * pIndex, CSphString & sError ) = 0;
+
+	/// truncate index (that is, kill all data)
+	virtual bool Truncate ( CSphString & sError ) = 0;
+
+	virtual void Optimize ( volatile bool * pForceTerminate, ThrottleState_t * pThrottle ) = 0;
+
+	/// check settings vs current and return back tokenizer and dictionary in case of difference
+	virtual bool IsSameSettings ( CSphReconfigureSettings & tSettings, CSphReconfigureSetup & tSetup, CSphString & sError ) const = 0;
+
+	/// reconfigure index by using new tokenizer, dictionary and index settings
+	/// current data got saved with current settings
+	virtual void Reconfigure ( CSphReconfigureSetup & tSetup ) = 0;
+
+	/// get disk chunk
+	virtual CSphIndex * GetDiskChunk ( int iChunk ) = 0;
 };
 
 /// initialize subsystem
 class CSphConfigSection;
-void sphRTInit ();
+void sphRTInit ( const CSphConfigSection & hSearchd, bool bTestMode );
 void sphRTConfigure ( const CSphConfigSection & hSearchd, bool bTestMode );
 bool sphRTSchemaConfigure ( const CSphConfigSection & hIndex, CSphSchema * pSchema, CSphString * pError );
 
@@ -69,7 +88,7 @@ bool sphRTSchemaConfigure ( const CSphConfigSection & hIndex, CSphSchema * pSche
 void sphRTDone ();
 
 /// RT index factory
-ISphRtIndex * sphCreateIndexRT ( const CSphSchema & tSchema, const char * sIndexName, DWORD uRamSize, const char * sPath, bool bKeywordDict );
+ISphRtIndex * sphCreateIndexRT ( const CSphSchema & tSchema, const char * sIndexName, int64_t iRamSize, const char * sPath, bool bKeywordDict );
 
 /// Get current txn index
 ISphRtIndex * sphGetCurrentIndexRT();
@@ -89,5 +108,5 @@ void sphReplayBinlog ( const SmallStringHash_T<CSphIndex*> & hIndexes, DWORD uRe
 #endif // _sphinxrt_
 
 //
-// $Id: sphinxrt.h 3125 2012-02-28 15:39:55Z shodan $
+// $Id: sphinxrt.h 4885 2015-01-20 07:02:07Z deogar $
 //
