@@ -1,10 +1,10 @@
 //
-// $Id: sphinxsearch.h 3087 2012-01-30 23:07:35Z shodan $
+// $Id: sphinxsearch.h 4929 2015-03-04 07:43:10Z tomat $
 //
 
 //
-// Copyright (c) 2001-2012, Andrew Aksyonoff
-// Copyright (c) 2008-2012, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -18,21 +18,28 @@
 
 #include "sphinx.h"
 #include "sphinxquery.h"
-
-//////////////////////////////////////////////////////////////////////////
-// PACKED HIT MACROS
-//////////////////////////////////////////////////////////////////////////
+#include "sphinxint.h"
 
 //////////////////////////////////////////////////////////////////////////
 
 /// term modifiers
 enum TermPosFilter_e
 {
+	TERM_POS_NONE = 0,
 	TERM_POS_FIELD_LIMIT = 1,
-	TERM_POS_FIELD_START,
-	TERM_POS_FIELD_END,
-	TERM_POS_FIELD_STARTEND,
-	TERM_POS_ZONES
+	TERM_POS_FIELD_START = 2,
+	TERM_POS_FIELD_END = 3,
+	TERM_POS_FIELD_STARTEND = 4,
+	TERM_POS_ZONES = 5,
+};
+
+
+/// decoder state saved at a certain offset
+struct SkiplistEntry_t
+{
+	SphDocID_t		m_iBaseDocid;		///< delta decoder docid base (aka docid infinum)
+	int64_t			m_iOffset;			///< offset in the doclist file (relative to the doclist start)
+	int64_t			m_iBaseHitlistPos;	///< delta decoder hitlist offset base
 };
 
 
@@ -43,9 +50,10 @@ public:
 	// setup by query parser
 	CSphString		m_sWord;		///< my copy of word
 	CSphString		m_sDictWord;	///< word after being processed by dict (eg. stemmed)
-	SphWordID_t		m_iWordID;		///< word ID, from dictionary
-	int				m_iTermPos;
+	SphWordID_t		m_uWordID;		///< word ID, from dictionary
+	TermPosFilter_e	m_iTermPos;
 	int				m_iAtomPos;		///< word position, from query
+	float			m_fBoost;		///< IDF keyword boost (multiplier)
 	bool			m_bExpanded;	///< added by prefix expansion
 	bool			m_bExcluded;	///< excluded by the query (rval to operator NOT)
 
@@ -53,9 +61,10 @@ public:
 	int				m_iDocs;		///< document count, from wordlist
 	int				m_iHits;		///< hit count, from wordlist
 	bool			m_bHasHitlist;	///< hitlist presence flag
+	CSphVector<SkiplistEntry_t>		m_dSkiplist;	///< skiplist for quicker document list seeks
 
 	// iterator state
-	CSphSmallBitvec m_dQwordFields;	///< current match fields
+	FieldMask_t m_dQwordFields;	///< current match fields
 	DWORD			m_uMatchHits;	///< current match hits count
 	SphOffset_t		m_iHitlistPos;	///< current position in hitlist, from doclist
 
@@ -64,9 +73,10 @@ protected:
 
 public:
 	ISphQword ()
-		: m_iWordID ( 0 )
-		, m_iTermPos ( 0 )
+		: m_uWordID ( 0 )
+		, m_iTermPos ( TERM_POS_NONE )
 		, m_iAtomPos ( 0 )
+		, m_fBoost ( 1.0f )
 		, m_bExpanded ( false )
 		, m_bExcluded ( false )
 		, m_iDocs ( 0 )
@@ -76,10 +86,11 @@ public:
 		, m_iHitlistPos ( 0 )
 		, m_bAllFieldsKnown ( false )
 	{
-		m_dQwordFields.Unset();
+		m_dQwordFields.UnsetAll();
 	}
 	virtual ~ISphQword () {}
 
+	virtual void				HintDocid ( SphDocID_t ) {}
 	virtual const CSphMatch &	GetNextDoc ( DWORD * pInlineDocinfo ) = 0;
 	virtual void				SeekHitlist ( SphOffset_t uOff ) = 0;
 	virtual Hitpos_t			GetNextHit () = 0;
@@ -89,7 +100,7 @@ public:
 	{
 		m_iDocs = 0;
 		m_iHits = 0;
-		m_dQwordFields.Unset();
+		m_dQwordFields.UnsetAll();
 		m_bAllFieldsKnown = false;
 		m_uMatchHits = 0;
 		m_iHitlistPos = 0;
@@ -100,13 +111,15 @@ public:
 /// term setup, searcher view
 class CSphQueryNodeCache;
 class ISphZoneCheck;
+struct CSphQueryStats;
 class ISphQwordSetup : ISphNoncopyable
 {
 public:
 	CSphDict *				m_pDict;
 	const CSphIndex *		m_pIndex;
 	ESphDocinfo				m_eDocinfo;
-	CSphMatch				m_tMin;
+	const CSphRowitem *		m_pMinRow;
+	SphDocID_t				m_uMinDocid;
 	int						m_iInlineRowitems;		///< inline rowitems count
 	int						m_iDynamicRowitems;		///< dynamic rowitems counts (including (!) inline)
 	int64_t					m_iMaxTimer;
@@ -114,11 +127,15 @@ public:
 	CSphQueryContext *		m_pCtx;
 	CSphQueryNodeCache *	m_pNodeCache;
 	mutable ISphZoneCheck *	m_pZoneChecker;
+	CSphQueryStats *		m_pStats;
+	mutable bool			m_bSetQposMask;
 
 	ISphQwordSetup ()
 		: m_pDict ( NULL )
 		, m_pIndex ( NULL )
 		, m_eDocinfo ( SPH_DOCINFO_NONE )
+		, m_pMinRow ( NULL )
+		, m_uMinDocid ( 0 )
 		, m_iInlineRowitems ( 0 )
 		, m_iDynamicRowitems ( 0 )
 		, m_iMaxTimer ( 0 )
@@ -126,6 +143,8 @@ public:
 		, m_pCtx ( NULL )
 		, m_pNodeCache ( NULL )
 		, m_pZoneChecker ( NULL )
+		, m_pStats ( NULL )
+		, m_bSetQposMask ( false )
 	{}
 	virtual ~ISphQwordSetup () {}
 
@@ -133,10 +152,8 @@ public:
 	virtual bool						QwordSetup ( ISphQword * pQword ) const = 0;
 };
 
-//////////////////////////////////////////////////////////////////////////
-
 /// generic ranker interface
-class ISphRanker
+class ISphRanker : public ISphExtra
 {
 public:
 	virtual						~ISphRanker () {}
@@ -198,5 +215,5 @@ public:
 #endif // _sphinxsearch_
 
 //
-// $Id: sphinxsearch.h 3087 2012-01-30 23:07:35Z shodan $
+// $Id: sphinxsearch.h 4929 2015-03-04 07:43:10Z tomat $
 //
